'''
socket extention api.
builds on the standart socket library and adds more advance functions,
making it easier to make more varied sockets.
'''
#attempt to import sug_sock
try:
	from sug_sock import *
except:
	print('[-] sug_sock library not found')
	exit(1)
'''
#attempt to import threads
try:
	from threading import *
except:
	print('[-] Thread library not found')
	exit(1)
'''
#generic debug function
def debug():
	print('debug function: ')
	print(dir('sug_sock')) 
	#print(dir(os))
	#print(os.getcwd())

#nice meme
def meme():
	print("nice")
#another debug function
def test():
	a = sugsock()
	print(str(a.ip))
	print(str(a.port))
	print(str(a.proc))
	print(str(a.type))
	a.addAttribute('name','boop')
	print(str(a.getAttribute('name')))
	a.changeAttribute('name' , 'doop')
	print(str(a.getAttribute('name')))
#debug()
test()


'''
Attribute class.
'''
	
class attr:
	# so a call for a sugsock object goes like sugsock(ip,port,protocol,type), enter 0 or nothing for defult
	def __init__(self,name,value):
		self.name = name
		self.value = value


'''
socket extention api.
builds on the standart socket library and adds more advance functions,
making it easier to make more varied sockets.

so the sugsock would contain a regular socket object, with easy functions
stuff in a sock:
	port
	ip
	protocol
	ip type

in addition to that sug sock will have an attributes module
this attributes will be able to hold an attribut name and data.
so if i want to store the socket or peer name i could store them on the socket attributes

so i need to make a protocol for this, so sockets will know what to except
'''
#attempt to import sockets
try:
	import socket
except:
	print("[-] socket library not found")
	exit(1)
#attempt to import threads
try:
    import threading
except:
	print("[-] threading library not found")
	exit(1)
#attempt to import attributes
try:
	from Attribute import *
except:
	print("[-] Attribute module not found")
	exit(1)
	

#sugsock class
class sugsock:
	# so a call for a sugsock object goes like sugsock(ip,port,protocol,type), enter 0 or nothing for defult
	def __init__(self, ip = '0' , port = 0 , proc = 'tcp',type = 'v4'):
		try:
			if proc == 'tcp':
				if type == 'v4':
					self.sock = socket.socket()
				elif type == 'v6':
					self.sock = socket.socket(socket.AF_INET6,socket.SOCK_STREAM)
			if proc == 'udp':
				if type == 'v4':
					self.sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
				elif type == 'v6':
					self.sock = socket.socket(socket.AF_INET6,socket.SOCK_DGRAM)
			#if the port and ip were added on creation
			if (ip !='0')and(port != 0):
				self.sock.bind(port)
			#create self properties
			self.attributes = []
			self.ip = ip
			self.port = port
			self.proc = proc
			self.type = type
			self.threads = True
		#what fucked up now
		except Exception as e:
			print("[-] an error has occured in creating the sugsock. recheck values")
			print(str(e))
		return		
	def setIp(self,ip):
		try:
			self.ip = ip
		except Exception as e:
			print("[-] an error has occured in assaigning ip to the sugsock. recheck values")
			print(str(e))	
	def bind(self,ip,port):
		try:
			self.sock.bind(port)
			self.ip = ip
			self.port = port
		except Exception as e:
			print("[-] an error has occured in binding the sugsock. recheck values")
			print(str(e))
		return
	def addAttribute(self,name,attribute):
		try:
			a = attr(name,attribute)
			self.attributes.append(a)
		except:
			print("[-] attribute addition has failled.")
		return
	def getAttribute(self,name):
		try:
			for x in range(0,len(self.attributes)):
				if self.attributes[x].name == name:
					val = self.attributes[x].value
					return val
			print("[-] getAttribute -  attribute to be returned not found.")
			return None
		except:
			print("[-] get attribute procces has failled")
			return None
	def changeAttribute(self,name,attribute):
		try:
			for x in range(0,len(self.attributes)):
				if self.attributes[x].name == name:
					self.attributes[x].value = attribute
					return 
			print("[-] changeAttribute - attribute to be changed not found.")
			return
		except:
			print("[-] change attribute procces has failled")
			return None
	#use threads to send and recive or not
	def background(self,state):
		try:
			if state == True:
				self.threads = True
			elif state == False:
				self.threads = False
		except:
			print("background function uses boolean")
		return
	#connect to external port send some stuff and disconnect
	def sendTo(self,Add,data):#############################################################
		try:
			if self.threads:
				t= Thread(target = senddata,args = (self,Add,data))
				t.start()
			else:
				senddata()
		except Exception as e:
			print("[-] an error has occured in sendTo")
			print(str(e))
		return
			
	'''def senddata(self,Add,data):
		sk = self.sock
		sk.connect()# --------------------------------------------------------------------------------------'''


#FIX DE THING
#might switch for asyncio if threads are not fitting

#use tcp for fail safe transport of info.
#use udp for fast transport of info.
#tcp - text, info, file sender.
#udp - voice chat, video chat.
#-----using "select" stuff for now, might switch for more purpose buit libarys or threads.
#using threads becuse select and asyncio are for python 3
import string
from threading import * 
#from socket import *
#import selectors
import socket
#def login():
IP = "0.0.0.0"
PORT = 1034
Ports = [1035,1036,1037,1038,1039]
#store logged in users
clients = []
#store an array of arrays, first object in array is the server name, next objects are usernames allowed in the server
servers=[]
#suppose to stop the program, dosnt work, fix it
STOP = True
######
#so this is my server api for now:
# A  /  B  /  C  /  D
#A - can be either 'private' , 'server' , 'req' depending on who the message is for or if this is a request.
#B - the reciver, who the message is intedent to.
#C - the type, is it a txt, data, or other kind of info being transsferred.
#D - the data itself.
class Server:
	def __init__(self,name,id = 0):
		self.allowed = []
		self.name = name
		temp = self.pickId()
		if temp == False:
			print("oh fuck")
		else:
			self.id = temp
		
	def addAllow(self,us):
		self.allowed.append(us)
		return
	def pickId(self):
		file = open("serverInfo.txt",'r')
		for line in file:
			nick = line.split("  :  ")[0]
			if nick == self.name:
				number = line.split("  :  ")[1]
				file.close()
				return number
		file.close()
		return False
	def sendData(self,val,whom,type):
		if type == "txt":
			for user in self.allowed:
				print("sending to: " + str(user.name)+" val: "+val)
				t = Thread(target = self.sendsTxt,args = (val,user,whom,))
				t.start()
		return
	def sendsTxt(self,val,user,whom):
		print(user.name +"   "+str(user.sock))
		if user.sock!=0:
			print("sending")		
			print(b"msg  /  server  /  "+ whom.name+b"  /  "+self.name +b"  /  txt  /  "+str(val))
			user.sock.send(b"msg  /  server  /  "+ whom.name+b"  /  "+self.name +b"  /  txt  /  "+str(val))
		return		
			
#user object
class User:
	def __init__(self,name,password,id,sock = 0):
		self.name = name
		self.password = password
		self.id = id
		self.sock = sock
		self.friends = []
		
	#set this user connection port
	#if sock = 0 user is offline
	def setSock(self,sock):
		self.sock = sock
		return
	def removeSock(self):
		self.sock = 0
		return
	def addFriend(self,friend):
		self.friends.append(friend)
		return
		
def gui():
	#gui
	root = Tk()
	#gui settings
	root.title("SugCord")
	root.geometry("200x100")
	app = Frame(root)
	app.grid()
	#text
	button1 = Button(app,text = "push me")
	button1.grid()
	button2 = Button(app)
	button2.grid()
	button2.configure(text = "weh")
	#start gui
	root.mainloop()
#check if the user name and password are correct with the users.txt
def validUser(Aname,Apassword):
	#get the users file
	us = open("users.txt",'r')
	try:
		#for every line in the file
		for line in us:
			#if the name = the name given
			name = line.split("  :  ")[0]
			if name == Aname:
				#if the password = the password given
				password = line.split("  :  ")[1]
				if password == Apassword:
					us.close()
					print("noice")
					#return the id
					return line.split("  :  ")[2]
	except:
		print("[-] username or password are wrong")
	#us.sock.close()
	return False
	
#make dis
def createUser():
	#maybe later
	return

#verifies the user logging in and puts the sock obj in the correct user obj
def addUser(sock):
	ok = False
	while not(ok):
		#request the user info
		sock.send(b"req  /  username")
		name = str(sock.recv(1024))
		#if got some sort of commend, do it.
		if name == 'create new user':
			createUser()
		if name == 'exit':
			us.sock.send("logout")
			us.sock.close()
			return
		#request the user password
		sock.send(b"req  /  password")
		password = str(sock.recv(1024))
		#check if its valid with the users.txt
		valid = validUser(name,password)
		#if its ok
		if valid!=False:
			#get the correct user id
			id = valid
			ok = True
	#get the correct user object by id
	us = getUser(id)
	if us == False:
		return
	#set this user sock as this current connection sock
	us.setSock(sock)
	#send succes report
	us.sock.send(b"req  /  login ok")
	#start the thread
	startThread(us)
	return

#sets up server listen socket
def setServer():
	lsock = socket.socket()
	lsock.bind((IP,PORT))
	return lsock
def testLoop(sock):
	while True:
		print 'nice'
		print(sock.recv(1024))
		sock.send('test')
#waits for new connections from clients	
def waitConn(sock):
	print(str(sock.getsockname()))
	while STOP:
		sock.listen(1)
		(cSock,cAdd) = sock.accept()
		print("connection from: "+str(cAdd))
		#testLoop(cSock)
		t = Thread(target = addUser,args = (cSock,))
		t.start()
	removeClients()#####change this
	return
#removes socket from clients array###################################
def removeClients():
	for us in clients:
		logout(us)
	return
#handle client requests	
def handleClients(us):
	lsock = us.sock
	while STOP:
		#if got data :
		val = lsock.recv(1024)
		if val != "logout":
			handleData(val,us)
		else:
			logout(us)
			return
	
#logs the user out.
def logout(us):
	try:
		if us.sock != 0:
			print (us.name +" is logging out.")
			#send the user the logout confirmation
			us.sock.send(b"logout")
			#close the socket
			us.sock.close()
			us.removeSock()
		else:
			print(us.name +" is already logged out.")
	except:
		print("error logging out")
	return
#starts the thread
def startThread(us):
	lsock = us.sock
	#this was suppose the send the log of everything that happened while the user was not online
	#file = open("log.txt",'r')
	#for line in file:
	#	lsock.send(line)
	#file.close()
	t= Thread(target = handleClients,args = (us,))
	t.start()
#checks what type of data it is and what to do with it.
#ok so data should come like this
#serv OR user  /  server OR user  /  type  /  val 
def handleData(val,us):
	mode = val.split("  /  ")[0]
	if mode == 'req':
		handleRequest(val,us)
		return
	mode,whom,type,data = val.split("  /  ")
	if type == "txt":
		#checkSend(whom,data,mode,us)
		sendText(whom,data,mode,us)
	if type == 'file':
		sendFile(val,us)
	return
def sendFile(val,us):
	User= ''
	for user in clients:
		#if the user in the clients array got the same name as the person the data is sent to.
		if user.name == whom:
			#send the data to that user.
			user.sock.send(b"msg  /  private  /  "+ us.name+b"  /  file  /  "+data)
			User = user
	try:
		mode,whom,type,data = val.split("  /  ")
		sock = us.sock
		sock.send(User.recive(1024))
		val = sock.recv(1024)
		if val.split("  /  ")[0] == 'req':
			if val.split("  /  ")[1] == 'openFileSocket':
				lsock = socket.socket()
				prt = Ports[0]
				ports.remove(prt)
				lsock.bind((IP,prt))
				msg = 'req  /  transferPort  /  '+str(prt))
				sock.send(msg)
				User.send(msg)
				lsock.recive()
			
			
	
	
#sends text to either servers or clients########################might change it from name based to id based after ill get the client interface done.
#whom - the reciver of info, data - the data to be sent , arrayPick - is it a server or a private client, us - the sender user object.
def sendText(whom,data,arrayPick,us):
	#if its for a private user
	if arrayPick =="private":
		#go over the clients array
		#checkSend(whom,data,arrayPick,us)
		#return
		if checkSend(whom,data,arrayPick,us):
			for user in clients:
				#if the user in the clients array got the same name as the person the data is sent to.
				if user.name == whom:
					#send the data to that user.
					#print("private message from " + us.name+": "+data)
					#user.sock.send(b"private message from " + us.name+b": "+data)
					user.sock.send(b"msg  /  private  /  "+ us.name+b"  /  txt  /  "+data)
					return
			#if the user was not found
			us.sock.send(b"[-] ERROR. User "+ whom+b" not found")
			return
	#if its for a server
	elif arrayPick == "server":
		#go over the servers array
		for serv in servers:
			#if the server in the servers array got the same name as the server the data is sent to.
			if serv.name == whom:
				#call the server send data function.
				serv.sendData(data,us,'txt')
				return
		#if the server was not found.
		us.sock.send(b"[-] ERROR. Server "+ whom+b" not found")
		return
def checkSend(whom , data , arrayPick , us):
	if arrayPick =="private":
		reciver = getUserByName(whom)
		for id in us.friends:
			if id = reciver.id:
				return True
		return False	
	return
#checks for admin input
def adminLoop():
	global STOP
	while(STOP):
		a = raw_input()
		if a == 'stop':
			STOP = False
			print("stop: " + str(STOP))
			removeClients()
		if a == 'users':
			arr = []
			for us in clients:
				if us.sock!= 0:
					arr.append(us.name)
			print (str(len(arr))+" "+str(arr))
	return
def handleRequest(val,us):
	mode,request = val.split("  /  ")
	if request.split(" ")[0] == 'getLog':
		print("boop")#################get the log of the user specified after the : and send it back
	if request.split(" ")[0] == 'getList':
		srv,prv = getAvailable(us)
		msg = 'req  /  reply  /  getAvailable  /  '+str(srv)+'  /  '+str(prv)
		us.sock.send(msg)
	return
def getAvailable(us):
	private = []
	serv = []
	friends = us.friends
	for friend in friends:
		cl = getUser(friend)
		clname = cl.name
		private.append(clname)
	for server in servers:
		for user in server.allowed:
			if user.id == us.id:
				serv.append(server.name)
	return serv,private
		
def sendLoop():
	logFile = open("log.txt",'r+')
	while(STOP):
		#if ya got something in the buffer
		if len(BUFFER)>0:####################################33gotta change alot here to make it send only to appropriate clients
			#go over all the clients
			for x in range(0,len(clients)):#might wanna switch things up here to fix end case probloms
				sk = clients[x].sock
				#send everything in buffer
				for y in range(0,len(BUFFER)):
					sk.send(BUFFER[y])		
			#remove objects from buffer
			for x in range(0,len(BUFFER)):
				logFile.write(BUFFER[x]+"\n")
				BUFFER.remove(BUFFER[x])
	for x in range(0,len(clients)):
		sk = clients[x]
		sk.send("sToPPP")
	file.close()
	return
#goes over the clients list and returns the user object with the correct id. returns False if none found.
def getUserByName(name):
	for x in range(0,len(clients)):
		us = clients[x]
		Name = us.name
		if Name == name:
			return us
		return False
def getUser(ID):
	#for x in the range of all clients
	for x in range(0,len(clients)):
		us = clients[x]
		id = us.id
		if id==ID:
			return us
	return False
	
#creates the users objects and stores them into the clients array.	
def getUSERS():
	#open the users info file
	file = open("users.txt",'r')
	#for every line in the users file
	for line in file:
		#put the data into the correct varables.
		name = line.split("  :  ")[0]
		password = line.split("  :  ")[1]
		id = line.split("  :  ")[2]
		friends = line.split("  :  ")[3]
		#create a user object
		us = User(name,password,id)
		#add friends
		for x in range(1,len(friends.split("  ,  "))+1):
			friend = line.split('  ,  ')[x]
			if x == len(friends.split("  ,  ")):
				friend = friend.split('\n')[0]
			us.addFriend(friend)
		#add the user object to the clients array.
		clients.append(us)
	return
#creates the servers objects and stores them into the servers array.	
def getServers():
	#open server info file
	file = open("serverInfo.txt",'r')
	#for every line in the server file
	for line in file:
		#put the data from the line into fitting varables.
		name = line.split("  :  ")[0]
		id = line.split("  :  ")[1]
		#create Server object
		se = Server(name,id)
		#go other all the users allowed on this specific server
		for x in range(1,len(line.split("  ,  "))):
			#put the allowed user id in a varable
			usr = line.split("  ,  ")[x]
			#get the correct user object from the clients array.
			us = getUser(usr)
			#if it exists
			if us!=False:
				#add the user object to the server allowed array.
				se.addAllow(us)
		#add the server to the servers list
		servers.append(se)
	return
#main
def main():
	#set up server
	sock = setServer()
	#start checking for admin orders
	s = Thread(target = adminLoop)
	s.start()
	#get users and put em into the client list
	getUSERS()
	#get servers and put em in the buffer
	getServers()
	#start looking for clients
	t = Thread(target = waitConn,args = (sock,))
	t.start()
	return

	

if __name__ == "__main__":
	main()
	


#from Tkinter import *

#add screen lock

#ok so igotta come up with an api for the client messages
import socket
import string
from threading import *
#def login():

VIEW = ''
Host = 'localhost'
def connect():
	sock = socket.socket()
	sock.connect((Host,1034))
	#sock.connect(("79.182.12.79",1034))
	#sock.connect(("10.0.0.2",1034))
	return sock
def gui():
	#gui
	root = Tk()
	#gui settings
	root.title("SugCord")
	root.geometry("200x100")
	app = Frame(root)
	app.grid()
	#text
	button1 = Button(app,text = "push me")
	button1.grid()
	button2 = Button(app)
	button2.grid()
	button2.configure(text = "weh")
	#start gui
	root.mainloop()
def viewData(sock):
	data = raw_input("what user do you want to view?: ")
	dataFile = open("clientData.txt" , 'r')
	for line in dataFile:
		if line.split("  /  ")[2] == data:
				message = "private message from "+data+": "+line.split("  /  ")[4] 
				print (message)
		if line.split("  /  ")[3] == data:
				message = "private message from "+line.split("  /  ")[2]+" at: "+data+": "+line.split("  /  ")[4] 
				print (message)
	return
		
def communicate(sock):
	try:
		while True:
			data = raw_input("")
			if data == 'view':#this could go wrong very easly
				viewData(sock)
			elif data == 'file':
				sendFile(sock)
			else:	
				sock.send(data)
			if data == "logout":
				print("logging out")
				return
	except:
		print("simething went wrong on the communucate function")
def handleRequest(val,sock):
	kind = val.split("  /  ")[0]
	what = val.split("  /  ")[1]
	if what == 'reply':
		type = val.split("  /  ")[2]
		if type == 'getAvailable':
			srv = val.split('  /  ')[3]
			prv = val.split('  /  ')[4]
			print('users available: ' +prv)
			print('servers available: ' +srv)
	if what == 'password':
		password = raw_input("enter password: ")
		sock.send(password)
	if what == 'username':
		username = raw_input("enter username: ")
		sock.send(username)
	if what == 'login ok':
		t = Thread(target = communicate, args = (sock,))
		t.start()
	if what == 'Error':
		error = val.split('  /  ')[2]
		print (error)
	return
def recivePrivate(val):
	kind,senderType,sender,type,data = val.split("  /  ")
	if type == "txt":
		log = open("clientData.txt",'a')
		header = val#make sure the message dosnt have a new line charachter in it
		print("appending")
		log.write(val+"  /  \n")
		log.close()
	if type == "file":
		t = Thread(target = handleFile,args = (val,))
		t.start()	
	return
def handleFile(val):
	kind,senderType,sender,type,data = val.split("  /  ")
	if VIEW == sender:
			print("incoming file from: "+sender+" " +data+". accept download?")
			answer = raw_input("yes to accept download, anything else to discard: ")
			if answer == 'yes':
				print ('downloading')
				download()
				
			else:
				print ('discarded')
	else:
		print('file from: ' + sender+ ' discarded.')
	return
def download():
	try:
		transferPort = socket.socket()
		transferPort.connect((Host,transferPort))
		RecivedFile= transferPort.recv(length)
		savedFile = ("downloadedfile" ,'w+')
		savedFile.write(RecivedFile)
		savedFile.close()
		transferPort.send("req  /  done")
		transferPort.close()
	except:
		print('download failed')
	return
def sendFile(sock):
	whom = raw_input("to: ")
	filename = raw_input("file name: ")
	msg = ('msg  /  '+whom+'  /  file  /  '+filename)
	sock.send(msg)
	answer = sock.recv(1024)
	if(answer.split("  /  ")[0] == 'req':
		if(answer.split("  /  ")[1] != 'yes':
			print('sending denied')
			return
	else:
		print('sending failed')
		return
	try:
		file = open(filename,'rb')
		sock.send("req  /  openFileSocket")
		val = sock.recv(1024)
		if(val.split("  /  ")[0] == 'req'):
			if(val.split("  /  ")[1] == 'transferPort'):
				Fport = val.split("  /  ")[2]
				length = val.split("  /  ")[3]
		transferPort = socket.socket()
		transferPort.connect((Host,transferPort))
	except:
		print('sending failed')
	return
	
def handleMessage(val):
	senderType = val.split("  /  ")[1]
	if(senderType == 'private'):
		recivePrivate(val)
	elif(senderType == 'server'):
		reciveServer(val)
	
def checkInfo(sock):
	while True:
		val =sock.recv(1024)
		kind = val.split("  /  ")[0]
		print val
		if kind == 'req':
			handleRequest(val,sock)
		if kind == 'msg':
			handleMessage(val)
		#print(val)
		if val == "logout":
			return
	
def main():
	#gui()
	sock = connect()
	#t = Thread(target = checkInfo , args = (sock,))
	#t.start()
	checkInfo(sock)
	#checkInfo()
	#communicate(sock)
	sock.close()
	return
	

if __name__ == "__main__":
	main()
	
	
	
	


'''
socket extention gui setup file
'''
from setuptools import setup


with open("README.md", "r") as fh:
    long_description = fh.read()
	

setup(name='sug_sock',
	  version='0.0.7',
      description='suguv socket expantion pack',
	  long_description=long_description,
	  long_description_content_type="text/markdown",
      url='https://bitbucket.org/SuguvHyde/sug_sock/src/master/',
      author='Suguv',
      author_email='orsegev7@gmail.com',
      license='MIT',
      packages=['sug_sock'],
      zip_safe=False)